/*
 * Copyright (C) 2020-2021  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef GENERIC_UNWRAPPERS_HH_99F7ED407368BF5047E91056503C0CC9//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define GENERIC_UNWRAPPERS_HH_99F7ED407368BF5047E91056503C0CC9

#include "src/impl/strong_type.hh"

#include <boost/optional.hpp>

#include <algorithm>
#include <type_traits>
#include <utility>
#include <vector>

namespace Fred {
namespace DbReport {
namespace Iface {
namespace Tools {

template <typename Type>
void unwrap(const Type& src, Type& dst);

template <typename Type>
void unwrap(const Type& src, boost::optional<Type>& dst);

template <typename SrcType, typename DstType>
void unwrap(const SrcType& src, boost::optional<DstType>& dst);

template <typename SrcType, typename DstType, typename DstTypeTag, template <typename> class DstTypeSkill>
std::enable_if_t<!std::is_same<decltype(std::declval<const SrcType&>().value()), void>::value>
unwrap(const SrcType& src, LibStrong::Type<DstType, DstTypeTag, DstTypeSkill>& dst);

template <typename SrcTypeVector, typename DstType>
std::enable_if_t<std::is_same<decltype(std::declval<const SrcTypeVector&>().size()), int>::value &&
                 std::is_pointer<std::remove_reference_t<decltype(*std::declval<const SrcTypeVector&>().data())>>::value>
unwrap(const SrcTypeVector& src, std::vector<DstType>& dst);


template <typename Type>
void unwrap(const Type& src, Type& dst)
{
    dst = src;
}

template <typename Type>
void unwrap(const Type& src, boost::optional<Type>& dst)
{
    dst = src;
}

template <typename SrcType, typename DstType>
void unwrap(const SrcType& src, boost::optional<DstType>& dst)
{
    DstType dst_value;
    unwrap(src, dst_value);
    dst = dst_value;
}

template <typename SrcType, typename DstType, typename DstTypeTag, template <typename> class DstTypeSkill>
std::enable_if_t<!std::is_same<decltype(std::declval<const SrcType&>().value()), void>::value>
unwrap(const SrcType& src, LibStrong::Type<DstType, DstTypeTag, DstTypeSkill>& dst)
{
    DstType dst_value;
    unwrap(src.value(), dst_value);
    dst = LibStrong::Type<DstType, DstTypeTag, DstTypeSkill>{dst_value};
}

template <typename SrcTypeVector, typename DstType>
std::enable_if_t<std::is_same<decltype(std::declval<const SrcTypeVector&>().size()), int>::value &&
                 std::is_pointer<std::remove_reference_t<decltype(*std::declval<const SrcTypeVector&>().data())>>::value>
unwrap(const SrcTypeVector& src, std::vector<DstType>& dst)
{
    dst.clear();
    dst.reserve(src.size());
    const auto* const* const src_begin = src.data();
    const auto* const* const src_end = src_begin + src.size();
    std::for_each(src_begin, src_end, [&](const auto* src_item) { DstType dst_value; unwrap(*src_item, dst_value); dst.emplace_back(std::move(dst_value)); });
}

}//namespace Fred::DbReport::Iface::Tools
}//namespace Fred::DbReport::Iface
}//namespace Fred::DbReport
}//namespace Fred

#endif//GENERIC_UNWRAPPERS_HH_99F7ED407368BF5047E91056503C0CC9
