/*
 * Copyright (C) 2020-2021  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef GENERIC_WRAPPERS_HH_E779CE0E6FCF6DAE8FADF8C55DB723BB//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define GENERIC_WRAPPERS_HH_E779CE0E6FCF6DAE8FADF8C55DB723BB

#include "src/impl/strong_type.hh"

#include <boost/optional.hpp>

#include <algorithm>
#include <type_traits>
#include <set>
#include <vector>

namespace Fred {
namespace DbReport {
namespace Iface {
namespace Tools {

template <typename SrcType, typename DstTypeGetter>
std::enable_if_t<std::is_pointer<decltype(std::declval<DstTypeGetter>()())>::value>
wrap(const boost::optional<SrcType>& src, DstTypeGetter getter);

template <typename SrcType, typename DstTypeVector>
std::enable_if_t<std::is_pointer<decltype(std::declval<DstTypeVector*>()->Add())>::value>
wrap(const std::vector<SrcType>& src, DstTypeVector* dst);

template <typename SrcType, typename SrcTypeCmp, typename DstTypeVector>
std::enable_if_t<std::is_pointer<decltype(std::declval<DstTypeVector*>()->Add())>::value>
wrap(const std::set<SrcType, SrcTypeCmp>& src, DstTypeVector* dst);

template <typename Type>
void wrap(const Type& src, Type* dst);

template <typename SrcType, typename SrcTypeTag, template <typename> class Skill, typename DstType>
std::enable_if_t<std::is_pointer<decltype(std::declval<DstType*>()->mutable_value())>::value>
wrap(const LibStrong::Type<SrcType, SrcTypeTag, Skill>& src, DstType* dst);

template <typename SrcType, typename DstTypeGetter>
std::enable_if_t<std::is_pointer<decltype(std::declval<DstTypeGetter>()())>::value>
wrap(const boost::optional<SrcType>& src, DstTypeGetter getter)
{
    if (src != boost::none)
    {
        wrap(*src, getter());
    }
}

template <typename SrcType, typename DstTypeVector>
std::enable_if_t<std::is_pointer<decltype(std::declval<DstTypeVector*>()->Add())>::value>
wrap(const std::vector<SrcType>& src, DstTypeVector* dst)
{
    dst->Reserve(static_cast<int>(src.size()));
    std::for_each(src.begin(), src.end(), [&](const SrcType& src_item) { wrap(src_item, dst->Add()); });
}

template <typename SrcType, typename SrcTypeCmp, typename DstTypeVector>
std::enable_if_t<std::is_pointer<decltype(std::declval<DstTypeVector*>()->Add())>::value>
wrap(const std::set<SrcType, SrcTypeCmp>& src, DstTypeVector* dst)
{
    dst->Reserve(static_cast<int>(src.size()));
    std::for_each(src.begin(), src.end(), [&](const SrcType& src_item) { wrap(src_item, dst->Add()); });
}

template <typename Type>
void wrap(const Type& src, Type* dst)
{
    *dst = src;
}

template <typename SrcType, typename SrcTypeTag, template <typename> class Skill, typename DstType>
std::enable_if_t<std::is_pointer<decltype(std::declval<DstType*>()->mutable_value())>::value>
wrap(const LibStrong::Type<SrcType, SrcTypeTag, Skill>& src, DstType* dst)
{
    wrap(*src, dst->mutable_value());
}

}//namespace Fred::DbReport::Iface::Tools
}//namespace Fred::DbReport::Iface
}//namespace Fred::DbReport
}//namespace Fred

#endif//GENERIC_WRAPPERS_HH_E779CE0E6FCF6DAE8FADF8C55DB723BB
