/*
 * Copyright (C) 2020-2021  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/iface/tools/tools.hh"

#include "liblog/liblog.hh"

namespace Fred {
namespace DbReport {
namespace Iface {
namespace Tools {

grpc::Status on_internal_server_error(const std::exception& e)
{
    LIBLOG_ERROR("InternalServerError caught: {}", e.what());
    return grpc::Status(grpc::StatusCode::UNKNOWN, "InternalServerError");
}

grpc::Status on_invalid_argument(const std::exception& e)
{
    LIBLOG_WARNING("InvalidArgument caught: {}", e.what());
    return grpc::Status(grpc::StatusCode::INVALID_ARGUMENT, "InvalidArgument", e.what());
}

grpc::Status on_failed_precondition(const std::exception& e)
{
    LIBLOG_WARNING("FailedPrecondition caught: {}", e.what());
    return grpc::Status(grpc::StatusCode::FAILED_PRECONDITION, "FailedPrecondition", e.what());
}

grpc::Status on_sql_error(const std::exception& e)
{
    LIBLOG_WARNING("SQL error: {}", e.what());
    return grpc::Status(grpc::StatusCode::UNKNOWN, "SQL error", e.what());
}

grpc::Status on_std_exception(const std::exception& e)
{
    LIBLOG_ERROR("std::exception caught: {}", e.what());
    return grpc::Status(grpc::StatusCode::UNKNOWN, "Unknown exception based on std::exception");
}

grpc::Status on_unknown_exception()
{
    LIBLOG_ERROR("unknown exception caught");
    return grpc::Status(grpc::StatusCode::UNKNOWN, "Unknown exception");
}

}//namespace Fred::DbReport::Iface::Tools
}//namespace Fred::DbReport::Iface
}//namespace Fred::DbReport
}//namespace Fred
