/*
 * Copyright (C) 2020-2021  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef COMMON_TYPES_HH_91019FEC9158D3F8DA5C29F3D56E7F88//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define COMMON_TYPES_HH_91019FEC9158D3F8DA5C29F3D56E7F88

#include "src/impl/strong_type.hh"

#include <string>
#include <vector>

namespace Fred {
namespace DbReport {
namespace Impl {

using ReportName = StrongType<std::string, struct ReportNameTag_, Skill::Comparable>; // <lower case SQL function name>, e.g. deleted_domains

using DataType = StrongType<std::string, struct DataTypeTag_, Skill::Comparable>; // <lower case SQL data type name>, e.g. bigint

using ParameterName = StrongType<std::string, struct ParameterNameTag_, Skill::Comparable>; // <lower case SQL function parameter name>

using JsonEncodedValue = StrongType<std::string, struct JsonEncodedValueTag_>;

struct InputParameterDescription
{
    DataType data_type;
    ParameterName name;
    friend bool operator==(const InputParameterDescription& lhs, const InputParameterDescription& rhs) noexcept;
};

struct OutputParameterDescription
{
    DataType data_type;
    ParameterName name;
    friend bool operator==(const OutputParameterDescription& lhs, const OutputParameterDescription& rhs) noexcept;
};

struct ReportInfo
{
    ReportName name;
    std::vector<InputParameterDescription> input_parameters;
    std::vector<OutputParameterDescription> output_parameters;
    friend bool operator==(const ReportInfo& lhs, const ReportInfo& rhs) noexcept;
};

struct Table
{
    struct FieldDescription
    {
        DataType data_type;
        std::string name; // does not have to be unique within the Header.field_descriptions
    };
    struct JsonEncodedFields
    {
        std::string fields;
    };
    std::vector<FieldDescription> field_descriptions;
    std::vector<JsonEncodedFields> rows; // each row is described by the same field_descriptions
};

}//namespace Fred::DbReport::Impl
}//namespace Fred::DbReport
}//namespace Fred

#endif//COMMON_TYPES_HH_91019FEC9158D3F8DA5C29F3D56E7F88
